@@ -25,7 +25,6 @@
 /* MAYBE I LIED... -jrb */
 
 #include <config.h>
-#include <gconf/gconf-client.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <sys/types.h>
@@ -40,6 +39,7 @@
 #include <gio/gio.h>
 #include <pwd.h>
 #include <X11/Xutil.h>
+#include <canberra-gtk.h>
 
 #include "screenshot-shadow.h"
 #include "screenshot-utils.h"
@@ -49,13 +49,13 @@
 
 #define SCREENSHOOTER_ICON "applets-screenshooter"
 
-#define GNOME_SCREENSHOT_GCONF  "/apps/gnome-screenshot"
-#define INCLUDE_BORDER_KEY      GNOME_SCREENSHOT_GCONF "/include_border"
-#define INCLUDE_POINTER_KEY     GNOME_SCREENSHOT_GCONF "/include_pointer"
-#define LAST_SAVE_DIRECTORY_KEY GNOME_SCREENSHOT_GCONF "/last_save_directory"
-#define BORDER_EFFECT_KEY       GNOME_SCREENSHOT_GCONF "/border_effect"
-#define DELAY_KEY               GNOME_SCREENSHOT_GCONF "/delay"
-
+#define GNOME_SCREENSHOT_SCHEMA "org.gnome.gnome-screenshot"
+#define INCLUDE_BORDER_KEY      "include-border"
+#define INCLUDE_POINTER_KEY     "include-pointer"
+#define INCLUDE_ICC_PROFILE     "include-icc-profile"
+#define LAST_SAVE_DIRECTORY_KEY "last-save-directory"
+#define BORDER_EFFECT_KEY       "border-effect"
+#define DELAY_KEY               "delay"
 
 enum
 {
@@ -96,12 +96,14 @@ static char *last_save_dir = NULL;
 static char *window_title = NULL;
 static char *temporary_file = NULL;
 static gboolean save_immediately = FALSE;
+static GSettings *settings = NULL;
 
 /* Options */
 static gboolean take_window_shot = FALSE;
 static gboolean take_area_shot = FALSE;
 static gboolean include_border = FALSE;
 static gboolean include_pointer = TRUE;
+static gboolean include_icc_profile = TRUE;
 static char *border_effect = NULL;
 static guint delay = 0;
 
@@ -114,6 +116,8 @@ static void  save_options           (void);
 static GtkWidget *border_check = NULL;
 static GtkWidget *effect_combo = NULL;
 static GtkWidget *effect_label = NULL;
+static GtkWidget *effects_vbox = NULL;
+static GtkWidget *delay_hbox = NULL;
 
 static void
 display_help (GtkWindow *parent)
@@ -168,6 +172,9 @@ target_toggled_cb (GtkToggleButton *button,
       gtk_widget_set_sensitive (border_check, take_window_shot);
       gtk_widget_set_sensitive (effect_combo, take_window_shot);
       gtk_widget_set_sensitive (effect_label, take_window_shot);
+
+      gtk_widget_set_sensitive (delay_hbox, !take_area_shot);
+      gtk_widget_set_sensitive (effects_vbox, !take_area_shot);
     }
 }
 
@@ -215,7 +222,7 @@ effect_combo_changed_cb (GtkComboBox *combo,
 static gint 
 key_press_cb (GtkWidget* widget, GdkEventKey* event, gpointer data)
 {
-  if (event->keyval == GDK_F1)
+  if (event->keyval == GDK_KEY_F1)
     {
       display_help (GTK_WINDOW (widget));
       return TRUE;
@@ -318,6 +325,7 @@ create_effects_frame (GtkWidget   *outer_vbox,
   main_vbox = gtk_vbox_new (FALSE, 6);
   gtk_box_pack_start (GTK_BOX (outer_vbox), main_vbox, FALSE, FALSE, 0);
   gtk_widget_show (main_vbox);
+  effects_vbox = main_vbox;
 
   title = g_strconcat ("<b>", frame_title, "</b>", NULL);
   label = gtk_label_new (title);
@@ -451,7 +459,7 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
 
   /** Grab area of the desktop **/
   radio = gtk_radio_button_new_with_mnemonic (group,
-                                              _("Grab a selected _area"));
+                                              _("Select _area to grab"));
   if (take_area_shot)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);
   g_signal_connect (radio, "toggled",
@@ -461,16 +469,16 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
   gtk_widget_show (radio);
 
   /** Grab after delay **/
-  hbox = gtk_hbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
+  delay_hbox = gtk_hbox_new (FALSE, 6);
+  gtk_box_pack_start (GTK_BOX (vbox), delay_hbox, FALSE, FALSE, 0);
+  gtk_widget_show (delay_hbox);
 
   /* translators: this is the first part of the "grab after a
    * delay of <spin button> seconds".
    */
   label = gtk_label_new_with_mnemonic (_("Grab _after a delay of"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (delay_hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
   adjust = GTK_ADJUSTMENT (gtk_adjustment_new ((gdouble) delay,
@@ -481,7 +489,7 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
   g_signal_connect (spin, "value-changed",
                     G_CALLBACK (delay_spin_value_changed_cb),
                     NULL);
-  gtk_box_pack_start (GTK_BOX (hbox), spin, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (delay_hbox), spin, FALSE, FALSE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin);
   gtk_widget_show (spin);
 
@@ -490,7 +498,7 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
    */
   label = gtk_label_new (_("seconds"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (delay_hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 }
 
@@ -499,25 +507,24 @@ create_interactive_dialog (void)
 {
   GtkWidget *retval;
   GtkWidget *main_vbox;
+  GtkWidget *content_area;
 
   retval = gtk_dialog_new ();
   gtk_window_set_resizable (GTK_WINDOW (retval), FALSE);
   gtk_container_set_border_width (GTK_CONTAINER (retval), 5);
-  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (retval)->vbox), 2);
+  content_area = gtk_dialog_get_content_area (GTK_DIALOG (retval));
+  gtk_box_set_spacing (GTK_BOX (content_area), 2);
   gtk_window_set_title (GTK_WINDOW (retval), _("Take Screenshot"));
 
   /* main container */
   main_vbox = gtk_vbox_new (FALSE, 18);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 5);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (retval)->vbox),
-                      main_vbox,
-                      TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (content_area), main_vbox, TRUE, TRUE, 0);
   gtk_widget_show (main_vbox);
 
   create_screenshot_frame (main_vbox, _("Take Screenshot"));
   create_effects_frame (main_vbox, _("Effects"));
 
-  gtk_dialog_set_has_separator (GTK_DIALOG (retval), FALSE);
   gtk_dialog_add_buttons (GTK_DIALOG (retval),
                           GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -541,21 +548,15 @@ create_interactive_dialog (void)
 }
 
 static void
-save_folder_to_gconf (ScreenshotDialog *dialog)
+save_folder_to_settings (ScreenshotDialog *dialog)
 {
-  GConfClient *gconf_client;
   char *folder;
 
-  gconf_client = gconf_client_get_default ();
-
   folder = screenshot_dialog_get_folder (dialog);
-  /* Error is NULL, as there's nothing we can do */
-  gconf_client_set_string (gconf_client,
-			   LAST_SAVE_DIRECTORY_KEY, folder,
-                           NULL);
+  g_settings_set_string (settings,
+                         LAST_SAVE_DIRECTORY_KEY, folder);
 
   g_free (folder);
-  g_object_unref (gconf_client);
 }
 
 static void
@@ -619,7 +620,7 @@ save_callback (TransferResult result,
 
   if (result == TRANSFER_OK)
     {
-      save_folder_to_gconf (dialog);
+      save_folder_to_settings (dialog);
       set_recent_entry (dialog);
       gtk_widget_destroy (toplevel);
       
@@ -647,8 +648,7 @@ save_callback (TransferResult result,
                                        GTK_DIALOG_DESTROY_WITH_PARENT,
                                        GTK_MESSAGE_ERROR,
                                        GTK_BUTTONS_OK,
-                                       _("Error while saving screenshot"),
-                                       NULL);
+                                       _("Error while saving screenshot"));
       /* translators: first %s is the file path, second %s is the VFS error */
       gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
                                                 _("Impossible to save the screenshot "
@@ -766,9 +766,111 @@ run_dialog (ScreenshotDialog *dialog)
                     dialog);
 }
 
+static void
+play_sound_effect (GdkWindow *window)
+{
+  ca_context *c;
+  ca_proplist *p = NULL;
+  int res;
+
+  c = ca_gtk_context_get ();
+
+  res = ca_proplist_create (&p);
+  if (res < 0)
+    goto done;
+
+  res = ca_proplist_sets (p, CA_PROP_EVENT_ID, "screen-capture");
+  if (res < 0)
+    goto done;
+
+  res = ca_proplist_sets (p, CA_PROP_EVENT_DESCRIPTION, _("Screenshot taken"));
+  if (res < 0)
+    goto done;
+
+  if (window != NULL)
+    {
+      res = ca_proplist_setf (p,
+                              CA_PROP_WINDOW_X11_XID,
+                              "%lu",
+                              (unsigned long) GDK_WINDOW_XID (window));
+      if (res < 0)
+        goto done;
+    }
+
+  ca_context_play_full (c, 0, p, NULL, NULL);
+
+ done:
+  if (p != NULL)
+    ca_proplist_destroy (p);
+
+}
+
+static gchar *
+get_profile_for_window (GdkWindow *window)
+{
+  gboolean ret = FALSE;
+  GDBusConnection *connection;
+  GError *error = NULL;
+  guint xid;
+  gchar *icc_profile = NULL;
+  GVariant *args = NULL;
+  GVariant *response = NULL;
+  GVariant *response_child = NULL;
+  GVariantIter *iter = NULL;
+
+  /* get a session bus connection */
+  connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
+  if (connection == NULL)
+    {
+      g_debug ("Failed to connect to session bus: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  /* get color profile */
+  xid = GDK_WINDOW_XID (window);
+  args = g_variant_new ("(u)", xid),
+  response = g_dbus_connection_call_sync (connection,
+                                          "org.gnome.ColorManager",
+                                          "/org/gnome/ColorManager",
+                                          "org.gnome.ColorManager",
+                                          "GetProfileForWindow",
+                                          args,
+                                          G_VARIANT_TYPE ("(s)"),
+                                          G_DBUS_CALL_FLAGS_NONE,
+                                          -1, NULL, &error);
+  if (response == NULL)
+    {
+      /* not a warning, as GCM might not be installed / running */
+      g_debug ("The GetProfileForWindow request failed: %s",
+               error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  /* get icc profile filename */
+  response_child = g_variant_get_child_value (response, 0);
+  icc_profile = g_variant_dup_string (response_child, NULL);
+out:
+  if (iter != NULL)
+    g_variant_iter_free (iter);
+  if (args != NULL)
+    g_variant_unref (args);
+  if (response != NULL)
+    g_variant_unref (response);
+  return icc_profile;
+}
+
 static void
 finish_prepare_screenshot (char *initial_uri, GdkWindow *window, GdkRectangle *rectangle)
 {  
+  char *icc_profile_filename;
+  guchar *icc_profile;
+  gsize icc_profile_size;
+  char *icc_profile_base64;
+  gboolean ret;
+  GError *error = NULL;
+
   ScreenshotDialog *dialog;
 
   /* always disable window border for full-desktop or selected-area screenshots */
@@ -803,9 +905,41 @@ finish_prepare_screenshot (char *initial_uri, GdkWindow *window, GdkRectangle *r
       exit (1);
     }
 
+  play_sound_effect (window);
+
   dialog = screenshot_dialog_new (screenshot, initial_uri, take_window_shot);
   g_free (initial_uri);
 
+  /* load ICC profile */
+  if (include_icc_profile)
+    {
+      icc_profile_filename = get_profile_for_window (window);
+      if (icc_profile_filename != NULL)
+        {
+          ret = g_file_get_contents (icc_profile_filename,
+                                     (gchar **) &icc_profile,
+                                     &icc_profile_size,
+                                     &error);
+          if (ret)
+            {
+              icc_profile_base64 = g_base64_encode (icc_profile,
+                                                    icc_profile_size);
+
+              /* use this profile for saving the image */
+              screenshot_set_icc_profile (icc_profile_base64);
+              g_free (icc_profile);
+              g_free (icc_profile_base64);
+            }
+          else
+            {
+              g_warning ("could not open ICC file: %s",
+                         error->message);
+              g_error_free (error);
+            }
+          g_free (icc_profile_filename);
+        }
+    }
+
   screenshot_save_start (screenshot, save_done_notification, dialog);
 
   run_dialog (dialog);
@@ -819,7 +953,10 @@ async_existence_job_free (AsyncExistenceJob *job)
 
   g_free (job->base_uris[1]);
   g_free (job->base_uris[2]);
-  g_free (job->rectangle);
+
+  if (job->rectangle != NULL)
+    g_slice_free (GdkRectangle, job->rectangle);
+
   g_slice_free (AsyncExistenceJob, job);
 }
 
@@ -1009,35 +1146,11 @@ find_current_window (char **window_title)
   return window;
 }
 
-static GdkRectangle *
-find_rectangle (void)
-{
-  GdkRectangle *rectangle;
-
-  if (!take_area_shot)
-    return NULL;
-
-  rectangle = g_new0 (GdkRectangle, 1);
-  if (screenshot_select_area (&rectangle->x, &rectangle->y,
-                              &rectangle->width, &rectangle->height))
-    {
-      g_assert (rectangle->width >= 0);
-      g_assert (rectangle->height >= 0);
-
-      return rectangle;
-    }
-  else
-    {
-      g_free (rectangle);
-      return NULL;
-    }
-}
-
 static void
-prepare_screenshot (void)
-{
+push_check_file_job (GdkRectangle *rectangle)
+{ 
   AsyncExistenceJob *job;
-  
+
   job = g_slice_new0 (AsyncExistenceJob);
   job->base_uris[0] = last_save_dir;
   /* we'll have to free these two */
@@ -1046,7 +1159,15 @@ prepare_screenshot (void)
   job->iteration = 0;
   job->type = TEST_LAST_DIR;
   job->window = find_current_window (&window_title);
-  job->rectangle = find_rectangle ();
+
+  if (rectangle != NULL)
+    {
+      job->rectangle = g_slice_new0 (GdkRectangle);
+      job->rectangle->x = rectangle->x;
+      job->rectangle->y = rectangle->y;
+      job->rectangle->width = rectangle->width;
+      job->rectangle->height = rectangle->height;
+    }
 
   /* Check if the area selection was cancelled */
   if (job->rectangle &&
@@ -1061,7 +1182,21 @@ prepare_screenshot (void)
                            job,
                            NULL,
                            0, NULL);
-                           
+}
+
+static void
+rectangle_found_cb (GdkRectangle *rectangle)
+{
+  push_check_file_job (rectangle);
+}
+
+static void
+prepare_screenshot (void)
+{
+  if (take_area_shot)
+    screenshot_select_area_async (rectangle_found_cb);
+  else
+    push_check_file_job (NULL);
 }
 
 static gboolean
@@ -1077,20 +1212,19 @@ prepare_screenshot_timeout (gpointer data)
 static gchar *
 get_desktop_dir (void)
 {
-  GConfClient *gconf_client;
+  GSettings *nautilus_prefs;
   gboolean desktop_is_home_dir = FALSE;
   gchar *desktop_dir;
 
-  gconf_client = gconf_client_get_default ();
-  desktop_is_home_dir = gconf_client_get_bool (gconf_client,
-                                               "/apps/nautilus/preferences/desktop_is_home_dir",
-                                               NULL);
+  nautilus_prefs = g_settings_new ("org.gnome.nautilus.preferences");
+  desktop_is_home_dir = g_settings_get_boolean (nautilus_prefs, "desktop-is-home-dir");
+
   if (desktop_is_home_dir)
     desktop_dir = g_strconcat ("file://", g_get_home_dir (), NULL);
   else
     desktop_dir = g_strconcat ("file://", g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP), NULL);
 
-  g_object_unref (gconf_client);
+  g_object_unref (nautilus_prefs);
 
   return desktop_dir;
 }
@@ -1129,14 +1263,9 @@ expand_initial_tilde (const char *path)
 static void
 load_options (void)
 {
-  GConfClient *gconf_client;
-
-  gconf_client = gconf_client_get_default ();
-
   /* Find various dirs */
-  last_save_dir = gconf_client_get_string (gconf_client,
-                                           LAST_SAVE_DIRECTORY_KEY,
-                                           NULL);
+  last_save_dir = g_settings_get_string (settings,
+                                         LAST_SAVE_DIRECTORY_KEY);
   if (!last_save_dir || !last_save_dir[0])
     {
       last_save_dir = get_desktop_dir ();
@@ -1148,46 +1277,35 @@ load_options (void)
       last_save_dir = tmp;
     }
 
-  include_border = gconf_client_get_bool (gconf_client,
-                                          INCLUDE_BORDER_KEY,
-                                          NULL);
+  include_border = g_settings_get_boolean (settings,
+                                           INCLUDE_BORDER_KEY);
 
-  include_pointer = gconf_client_get_bool (gconf_client,
-                                          INCLUDE_POINTER_KEY,
-                                          NULL);
+  include_icc_profile = g_settings_get_boolean (settings,
+                                                INCLUDE_ICC_PROFILE);
 
-  border_effect = gconf_client_get_string (gconf_client,
-                                           BORDER_EFFECT_KEY,
-                                           NULL);
+  include_pointer = g_settings_get_boolean (settings,
+                                            INCLUDE_POINTER_KEY);
+
+  border_effect = g_settings_get_string (settings,
+                                         BORDER_EFFECT_KEY);
   if (!border_effect)
     border_effect = g_strdup ("none");
 
-  delay = gconf_client_get_int (gconf_client, DELAY_KEY, NULL);
-
-  g_object_unref (gconf_client);
+  delay = g_settings_get_int (settings, DELAY_KEY);
 }
 
 static void
 save_options (void)
 {
-  GConfClient *gconf_client;
-
-  gconf_client = gconf_client_get_default ();
-
-  /* Error is NULL, as there's nothing we can do */
-
-  gconf_client_set_bool (gconf_client,
-                         INCLUDE_BORDER_KEY, include_border,
-                         NULL);
-  gconf_client_set_bool (gconf_client,
-                         INCLUDE_POINTER_KEY, include_pointer,
-                         NULL);
-  gconf_client_set_int (gconf_client, DELAY_KEY, delay, NULL);
-  gconf_client_set_string (gconf_client,
-                           BORDER_EFFECT_KEY, border_effect,
-                           NULL);
- 
-  g_object_unref (gconf_client);
+  g_settings_set_boolean (settings,
+                          INCLUDE_BORDER_KEY, include_border);
+  g_settings_set_boolean (settings,
+                          INCLUDE_POINTER_KEY, include_pointer);
+  g_settings_set_boolean (settings,
+                          INCLUDE_ICC_PROFILE, include_icc_profile);
+  g_settings_set_int (settings, DELAY_KEY, delay);
+  g_settings_set_string (settings,
+                         BORDER_EFFECT_KEY, border_effect);
 }
 
 
@@ -1225,7 +1343,6 @@ int
 main (int argc, char *argv[])
 {
   GOptionContext *context;
-  GOptionGroup *group;
   gboolean window_arg = FALSE;
   gboolean area_arg = FALSE;
   gboolean include_border_arg = FALSE;
@@ -1279,6 +1396,7 @@ main (int argc, char *argv[])
   gtk_window_set_default_icon_name (SCREENSHOOTER_ICON);
   screenshooter_init_stock_icons ();
 
+  settings = g_settings_new ("org.gnome.gnome-screenshot");
   load_options ();
   /* allow the command line to override options */
   if (window_arg)
@@ -1325,7 +1443,8 @@ main (int argc, char *argv[])
         }
     }
 
-  if ((delay > 0 && interactive_arg) || delay_arg > 0)
+  if (((delay > 0 && interactive_arg) || delay_arg > 0) &&
+      !take_area_shot)
     {      
       g_timeout_add (delay * 1000,
 		     prepare_screenshot_timeout,
